@@ -29,6 +29,7 @@
 #include <bonobo/bonobo-moniker-util.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomevfs/gnome-vfs.h>
+#include <libedataserver/e-util.h>
 #include <libedataserver/e-xml-hash-utils.h>
 #include <libecal/e-cal-recur.h>
 #include <libecal/e-cal-time-util.h>
@@ -43,6 +44,7 @@
 typedef struct {
 	ECalComponent *full_object;
 	GHashTable *recurrences;
+	GList *recurrences_list;
 } ECalBackendFileObject;
 
 /* Private part of the ECalBackendFile structure */
@@ -103,6 +105,9 @@ free_object (gpointer key, gpointer value, gpointer data)
 	g_object_unref (obj_data->full_object);
 	g_hash_table_foreach (obj_data->recurrences, (GHFunc) free_recurrence, NULL);
 	g_hash_table_destroy (obj_data->recurrences);
+	g_list_free (obj_data->recurrences_list);
+
+	g_free (obj_data);
 }
 
 /* Saves the calendar data */
@@ -367,22 +372,6 @@ check_dup_uid (ECalBackendFile *cbfile, ECalComponent *comp)
 	save (cbfile);
 }
 
-static const char *
-get_rid_string (ECalComponent *comp)
-{
-        ECalComponentRange range;
-        struct icaltimetype tt;
-                                                                                   
-        e_cal_component_get_recurid (comp, &range);
-        if (!range.datetime.value)
-                return "0";
-        tt = *range.datetime.value;
-        e_cal_component_free_range (&range);
-                                                                                   
-        return icaltime_is_valid_time (tt) && !icaltime_is_null_time (tt) ?
-                icaltime_as_ical_string (tt) : "0";
-}
-
 static struct icaltimetype
 get_rid_icaltime (ECalComponent *comp)
 {
@@ -423,13 +412,15 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 			return;
 		}
 
-		rid = get_rid_string (comp);
+		rid = e_cal_component_get_recurid_as_string (comp);
 		if (g_hash_table_lookup (obj_data->recurrences, rid)) {
 			g_warning (G_STRLOC ": Tried to adding an already existing recurrence");
 			return;
 		}
 
 		g_hash_table_insert (obj_data->recurrences, g_strdup (rid), comp);
+		/* FIXME: sort the recurrences */
+		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
 	} else {
 		/* Ensure that the UID is unique; some broken implementations spit
 		 * components with duplicated UIDs.
@@ -796,10 +787,20 @@ reload_cal (ECalBackendFile *cbfile, const char *uristr)
 static ECalBackendSyncStatus
 create_cal (ECalBackendFile *cbfile, const char *uristr)
 {
+	char *dirname;
 	ECalBackendFilePrivate *priv;
 
 	priv = cbfile->priv;
 
+	/* Create the directory to contain the file */
+	dirname = g_path_get_dirname (uristr);
+	if (e_util_mkdir_hier (dirname, 0700) != 0) {
+		g_free (dirname);
+		return GNOME_Evolution_Calendar_NoSuchCal;
+	}
+
+	g_free (dirname);
+
 	/* Create the new calendar information */
 	priv->icalcomp = e_cal_util_new_top_level ();
 
@@ -867,7 +868,11 @@ e_cal_backend_file_remove (ECalBackendSync *backend, EDataCal *cal)
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
-	char *str_uri;
+	char *str_uri, *dirname;
+	const char *fname;
+	GDir *dir;
+	GError *error = NULL;
+	gboolean success;
 	
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
@@ -882,16 +887,40 @@ e_cal_backend_file_remove (ECalBackendSync *backend, EDataCal *cal)
 		return GNOME_Evolution_Calendar_PermissionDenied;
 	}
 
-	/* FIXME Remove backup file and whole directory too? */
-	if (unlink (str_uri) != 0) {
+	/* remove all files in the directory */
+	dirname = g_path_get_dirname (str_uri);
+	dir = g_dir_open (dirname, 0, &error);
+	if (!dir) {
 		g_free (str_uri);
+		g_free (dirname);
 
-		return GNOME_Evolution_Calendar_OtherError;
+		return GNOME_Evolution_Calendar_PermissionDenied;
 	}
-	
+
+	while ((fname = g_dir_read_name (dir))) {
+		char *full_path;
+
+		full_path = g_build_filename (dirname, fname, NULL);
+		if (unlink (full_path) != 0) {
+			g_free (full_path);
+			g_free (str_uri);
+			g_free (dirname);
+			g_dir_close (dir);
+
+			return GNOME_Evolution_Calendar_OtherError;
+		}
+
+		g_free (full_path);
+	}
+
+	/* remove the directory itself */
+	success = rmdir (dirname) == 0;
+		
+	g_dir_close (dir);
 	g_free (str_uri);
-	
-	return GNOME_Evolution_Calendar_Success;
+	g_free (dirname);
+
+	return success ? GNOME_Evolution_Calendar_Success : GNOME_Evolution_Calendar_OtherError;
 }
 
 /* is_loaded handler for the file backend */
@@ -1610,7 +1639,7 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 	/* check the object is not in our cache */
 	if (lookup_component (cbfile, comp_uid)) {
 		icalcomponent_free (icalcomp);
-		return GNOME_Evolution_Calendar_CardIdAlreadyExists;
+		return GNOME_Evolution_Calendar_ObjectIdAlreadyExists;
 	}
 
 	/* Create the cal component */
@@ -1640,7 +1669,7 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, const
 
 static ECalBackendSyncStatus
 e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const char *calobj, 
-				CalObjModType mod, char **old_object)
+				  CalObjModType mod, char **old_object)
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
@@ -1680,7 +1709,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	/* Create the cal component */
 	comp = e_cal_component_new ();
 	e_cal_component_set_icalcomponent (comp, icalcomp);
-	
+
 	/* Set the last modified time on the component */
 	current = icaltime_from_timet (time (NULL), 0);
 	e_cal_component_set_last_modified (comp, &current);
@@ -1691,7 +1720,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 	/* handle mod_type */
 	switch (mod) {
 	case CALOBJ_MOD_THIS :
-		rid = get_rid_string (comp);
+		rid = e_cal_component_get_recurid_as_string (comp);
 		if (!rid || !*rid) {
 			g_object_unref (comp);
 			return GNOME_Evolution_Calendar_ObjectNotFound;
@@ -1699,11 +1728,15 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 		if (g_hash_table_lookup_extended (obj_data->recurrences, rid,
 						  &real_rid, &recurrence)) {
+			if (old_object)
+				*old_object = e_cal_component_get_as_string (recurrence);
+
 			/* remove the component from our data */
 			icalcomponent_remove_component (priv->icalcomp,
 							e_cal_component_get_icalcomponent (recurrence));
 			priv->comp = g_list_remove (priv->comp, recurrence);
 			g_hash_table_remove (obj_data->recurrences, rid);
+			obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, recurrence);
 
 			/* free memory */
 			g_free (real_rid);
@@ -1721,12 +1754,18 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 			e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old, new);
 
-			g_free (old);
+			if (old_object)
+				*old_object = old;
+			else
+				g_free (old);
 			g_free (new);
 		}
 
 		/* add the detached instance */
-		g_hash_table_insert (obj_data->recurrences, g_strdup (get_rid_string (comp)), comp);
+		g_hash_table_insert (obj_data->recurrences, 
+				     g_strdup (e_cal_component_get_recurid_as_string (comp)),
+				     comp);
+		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
 		break;
@@ -1736,6 +1775,9 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 		/* in this case, we blow away all recurrences, and start over
 		   with a clean component */
 		/* Remove the old version */
+		if (old_object)
+			*old_object = e_cal_component_get_as_string (obj_data->full_object);
+
 		remove_component (cbfile, obj_data->full_object);
 
 		/* Add the new object */
@@ -1745,9 +1787,6 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 
 	save (cbfile);
 
-	if (old_object)
-		*old_object = e_cal_component_get_as_string (comp);
-
 	return GNOME_Evolution_Calendar_Success;
 }
 
@@ -1767,6 +1806,7 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 						e_cal_component_get_icalcomponent (comp));
 		cbfile->priv->comp = g_list_remove (cbfile->priv->comp, comp);
 		g_hash_table_remove (obj_data->recurrences, rid);
+		obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, comp);
 
 		/* update the set of categories */
 		e_cal_component_get_categories_list (comp, &categories);
@@ -1821,6 +1861,8 @@ remove_object_instance_cb (gpointer key, gpointer value, gpointer user_data)
 							e_cal_component_get_icalcomponent (instance));
 			rrdata->cbfile->priv->comp = g_list_remove (rrdata->cbfile->priv->comp, instance);
 
+			rrdata->obj_data->recurrences_list = g_list_remove (rrdata->obj_data->recurrences_list, instance);
+
 			/* update the set of categories */
 			e_cal_component_get_categories_list (instance, &categories);
 			e_cal_backend_unref_categories (E_CAL_BACKEND (rrdata->cbfile), categories);
@@ -2003,7 +2045,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		case ICAL_VJOURNAL_COMPONENT:
 			tzdata.found = TRUE;
 			icalcomponent_foreach_tzid (subcomp, check_tzids, &tzdata);
-			
+
 			if (!tzdata.found) {
 				status = GNOME_Evolution_Calendar_InvalidObject;
 				goto error;
@@ -2026,6 +2068,9 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 
 	/* Now we manipulate the components we care about */
 	for (l = comps; l; l = l->next) {
+		const char *uid, *rid;
+		char *calobj;
+
 		subcomp = l->data;
 	
 		/* Create the cal component */
@@ -2040,13 +2085,35 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		/* sanitize the component*/
 		sanitize_component (cbfile, comp); 
 
+		e_cal_component_get_uid (comp, &uid);
+		rid = e_cal_component_get_recurid_as_string (comp);
+
 		switch (method) {
 		case ICAL_METHOD_PUBLISH:
 		case ICAL_METHOD_REQUEST:
-			/* FIXME Need to see the new create/modify stuff before we set this up */
-			break;			
 		case ICAL_METHOD_REPLY:
-			/* FIXME Update the status of the user, if we are the organizer */
+			if (e_cal_backend_file_get_object (backend, cal, uid, rid, &calobj)
+			    == GNOME_Evolution_Calendar_Success) {
+				char *returned_uid;
+
+				calobj = (char *) icalcomponent_as_ical_string (subcomp);
+				status = e_cal_backend_file_create_object (backend, cal, calobj, &returned_uid);
+				if (status != GNOME_Evolution_Calendar_Success)
+					goto error;
+
+				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), calobj);
+			} else {
+				char *old_object;
+
+				calobj = (char *) icalcomponent_as_ical_string (subcomp);
+				status = e_cal_backend_file_modify_object (backend, cal, calobj, CALOBJ_MOD_THIS, &old_object);
+				if (status != GNOME_Evolution_Calendar_Success)
+					goto error;
+
+				e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old_object, calobj);
+
+				g_free (old_object);
+			}
 			break;
 		case ICAL_METHOD_ADD:
 			/* FIXME This should be doable once all the recurid stuff is done */
@@ -2060,10 +2127,13 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 			goto error;
 			break;
 		case ICAL_METHOD_CANCEL:
-			/* FIXME Do we need to remove the subcomp so it isn't merged? */
 			if (cancel_received_object (cbfile, subcomp)) {
-				const char *calobj = icalcomponent_as_ical_string (subcomp);
+				calobj = (char *) icalcomponent_as_ical_string (subcomp);
 				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), icalcomponent_get_uid (subcomp), calobj);
+
+				/* remove the component from the toplevel VCALENDAR */
+				icalcomponent_remove_component (toplevel_comp, subcomp);
+				icalcomponent_free (subcomp);
 			}
 			break;
 		default:
@@ -2071,6 +2141,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 			goto error;
 		}
 	}
+
 	g_list_free (comps);
 	
 	/* Merge the iCalendar components with our existing VCALENDAR,
